# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# Copyright 2012 Thomi Richards

from gi.repository import GdkPixbuf
import os
import os.path
import logging


_pixbuf_cache = {}
_icon_loader = None

try:
    from gi.repository import Gtk
    _icon_loader = Gtk.IconTheme()
except RuntimeError:
    # raised when X11 not available.
    pass


def _cache_make_key(name, size):
    return name + str(size)


def cache_has_image_at_size(name, size):
    """Return true if the pixbuf cache has the image specified at the size specified."""
    key = _cache_make_key(name, size)
    return _pixbuf_cache.has_key(key)


def cache_get_image_at_size(name, size):
    """Return the image from the cache at the given size.

    KeyError is raised if the image is not already in the cache.
    """
    if not cache_has_image_at_size(name, size):
        raise KeyError("Image cache does not contain image '%s' at size '%d'" % (name, size))
    key = _cache_make_key(name, size)
    return _pixbuf_cache[key]


def cache_set_image_at_size(name, size, pixbuf):
    """Store an image in the cache."""
    if type(pixbuf) is not GdkPixbuf.Pixbuf:
        raise TypeError("Can only store GdkPixbuf.Pixbuf objects in the cache")
    key = _cache_make_key(name, size)
    _pixbuf_cache[key] = pixbuf


def system_has_image(icon_name):
    """Is the icon with the given name in the system icon theme?"""
    if _icon_loader:
        return _icon_loader.has_icon(icon_name)
    else:
        return False


def system_get_image(name, size):
    """Get an image from the system icon theme.

    IOError is raised if the image is not present in the current system icon theme.

    """
    if not system_has_image(name):
        raise IOError("Icon '%s' not found in system icon theme." % (name))
    if not _icon_loader:
        raise RuntimeError("Gtk IconTheme not available.")
    pb = _icon_loader.load_icon(name, size, 0)
    logging.debug("Loading pixbuf %s with size %d from system icons", name, size)
    return pb


def local_store_get_folder():
    return os.path.join(os.path.dirname(__file__), 'images')


def local_store_has_image(name):
    """Return true if the local disk has the image specified.

    Returns false if the 'images' folder does not exist, if the specified filename
    does not exist (after appending ".svg"), or the specified file is not readable.

    """
    filename = os.path.join(local_store_get_folder(), name + '.svg')
    return os.path.exists(filename) and os.access(filename, os.R_OK)


def local_store_get_image(name, size):
    """Get the image from the local directory.

    IOError is raised if the file is not found.
    """
    if not local_store_has_image(name):
        raise IOError("File '%s' not found in local store.")

    filename = os.path.join(local_store_get_folder(), name + '.svg')
    pb = GdkPixbuf.Pixbuf.new_from_file_at_size(filename, size, size)
    logging.debug("loading pixbuf %s with size %d from source directory", name, size)
    return pb


def load_pixbuf_by_name(name, size):
    """Load a pixbuf by name from several sources.

    First, an internal cache will be queried. If the image has already been loaded
    in the past, it can be retrieved from this cache instead of loading it from
    disk again.

    Second, the system icon theme is queried. If the image is present in the
    current icon theme, it will be loaded.

    Third, the 'images' folder below the location of graphics.py is used.

    Finally, if all else fails, a RuntimeError is raised.
    """
    if cache_has_image_at_size(name, size):
        return cache_get_image_at_size(name, size)

    if system_has_image(name):
        pb = system_get_image(name, size)
        cache_set_image_at_size(name, size, pb)
        return pb

    if local_store_has_image(name):
        pb =  local_store_get_image(name, size)
        cache_set_image_at_size(name, size, pb)
        return pb

    raise RuntimeError("Could not find image:", name)


def get_pixbuf_for_health_score(score, size):
    """Get an Image representing a health score between 0 and 100."""
    if type(score) is not int:
        raise TypeError("Score must be an integer value.")
    if score < 0 or score > 100:
        raise ValueError("Score must be between 0 and 100 (incl).")

    if score < 20:
        return load_pixbuf_by_name("health-00to19", size)
    elif score < 40:
        return load_pixbuf_by_name("health-20to39", size)
    elif score < 60:
        return load_pixbuf_by_name("health-40to59", size)
    elif score < 80:
        return load_pixbuf_by_name("health-60to79", size)
    else:
        return load_pixbuf_by_name("health-80plus", size)


def get_pixbuf_for_state(state_name, size):
    """Get a pixbuf representing the given state name.

    If the state name is unknown you will get the pixbug for the 'grey' state.

    """
    if state_name == 'blue':
        state_name = 'green'

    if state_name not in ('green', 'yellow', 'red', 'grey'):
        logging.warning("Unkown state name '%s', using 'grey' instead.", state_name)
        state_name = 'grey'
    return load_pixbuf_by_name('status-' + state_name, size)


def combine_two_images(left_pb, right_pb):
    """Combine two images side by side.

    Both images must be the same height. Returns a Gtk.Gdk.Pixmap object

    """
    if type(left_pb) is not GdkPixbuf.Pixbuf:
        raise TypeError("Left image must be a GdkPixbuf.Pixbuf")
    if type(right_pb) is not GdkPixbuf.Pixbuf:
        raise TypeError("Right image must be a GdkPixbuf.Pixbuf")

    if left_pb.get_height() != right_pb.get_height():
        raise ValueError("Left and Right images must be the same height.")

    if left_pb.get_width() != right_pb.get_width():
        raise ValueError("Left and Right images must be the same width.")

    rwidth = left_pb.get_width() + right_pb.get_width()
    result = GdkPixbuf.Pixbuf.new(right_pb.get_colorspace(),
                            right_pb.get_has_alpha(),
                            right_pb.get_bits_per_sample(),
                            rwidth,
                            right_pb.get_height())
    left_pb.copy_area(0,0,
                    left_pb.get_width(), left_pb.get_height(),
                    result,
                    0,0)
    right_pb.copy_area(0,0,
                    right_pb.get_width(), right_pb.get_height(),
                    result,
                    left_pb.get_width(), 0)
    return result
