# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# Copyright 2012 Thomi Richards


from gi.repository import Gtk
import logging
import webbrowser

from graphics import load_pixbuf_by_name, get_pixbuf_for_health_score, combine_two_images, get_pixbuf_for_state

logger = logging.getLogger(__name__)

class JenkinsJobMenuItem(Gtk.ImageMenuItem):
    """A menu item that has an image to represent the job state.

    We wrap a JenkinsJobModel and subscribe to it's property change notifications
    so we can update our appearance when the job changes.

    This class needs to work around a bug in appindicator:
        https://bugs.launchpad.net/libappindicator/+bug/934574

    """

    def __init__(self, label_text):
        super(JenkinsJobMenuItem, self).__init__(label_text)
        self._last_state_color = None
        self._last_health_score = None
        self.connect("activate", self.on_activated)

    def attach_job_model(self, job_model):
        self.model_object = job_model
        self.model_object.connect("notify::color", self.on_model_changed)
        self.model_object.connect("notify::health-description", self.on_model_changed)
        self.model_object.connect("notify::health-score", self.on_model_changed)
        self.set_label(self.model_object.name)
        if self.model_object.color == "":
            self.set_state_color("grey")
        else:
            self.set_state_color(self.model_object.color)

    def set_state_color(self, color_name):
        if color_name == self._last_state_color:
            return

        self._last_state_color = color_name
        self._set_graphic()

    def set_health_score(self, score):
        if self._last_health_score == score:
            return
        self._last_health_score = score
        # See TODO note below.
        #self._set_graphic()

    def _set_graphic(self):

        img = None

        if self._last_state_color is not None:
            img = get_pixbuf_for_state(self._last_state_color, 24)

        # TODO: Right now we cannot set non-square images in the appindicator menu
        # since they get scaled to be square and look awful. If that ever gets
        # fixed or changed we can uncomment this code to add the weather graphic
        # to the menu.
        #
        # if self._last_health_score is not None:
        #     health_img = get_pixbuf_for_health_score(self._last_health_score, 24)
        #     img = combine_two_images(img, health_img)

        if img is not None:
            i = Gtk.Image()
            i.set_from_pixbuf(img)
            self.set_image(i)
            self.set_always_show_image(True)

    def on_model_changed(self, sender, prop):
        logger.debug("Object %r changed property %s", sender, prop.name)
        if prop.name == 'color':
            self.set_state_color(self.model_object.color)
        if prop.name == 'health-description':
            tooltip = self.model_object.health_description
            self.set_tooltip_text(tooltip)
        if prop.name == 'health-score':
            self.set_health_score(self.model_object.health_score)

    def on_activated(self, sender):
        if self.model_object:
            webbrowser.open(self.model_object.url)

    def __repr__(self):
        return "<JenkinsJobMenuItem '%s','%s'>" % (self.get_label(), self._last_state_color)

