# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# Copyright 2012 Thomi Richards

from gi.repository import Gtk, GLib
import jenkins
import logging
from multiprocessing import Process, get_logger, Event
from multiprocessing.queues import SimpleQueue

logger = get_logger()
root_logger = logging.getLogger()
logger.setLevel(root_logger.getEffectiveLevel())
logger.handlers = root_logger.handlers[:]

# We probably only ever want to create one of these, so we do it here, and return
# references to it using the functions below...
__SERVER_MONITOR = None

def get_server_monitor():
    """Get a server monitor instance."""
    global __SERVER_MONITOR
    if __SERVER_MONITOR is None:
        __SERVER_MONITOR = _JenkinsServerMonitor()
        __SERVER_MONITOR.start()
    return __SERVER_MONITOR


# comm codes for the worker process:
_GET_SERVER_DATA = 1
_GET_JOB_DATA = 2

class _JenkinsServerMonitor(Process):
    """Can retrieve data from a jenkins server in a threaded manner."""

    def __init__(self):
        super(_JenkinsServerMonitor, self).__init__()
        self._input = SimpleQueue()
        self._output = SimpleQueue()
        self._quit_event = Event()
        self._cached_data = None
        self._callback_queue = []
        # TODO - if we MUST poll for data, instead of running it all the time,
        # only run it while there's an active request.
        GLib.timeout_add_seconds(1, self._poll_for_data)

    def start_get_server_data(self, server_url, callback):
        """Start getting server data from 'server_url'."""

        if not self.is_alive():
            logger.warning("Process is not running!")
        logger.debug("Got request for job data for server: %s", server_url)
        self._enqueue_command(_GET_SERVER_DATA,
                            [server_url],
                            callback)

    def start_get_job_data(self, server_url, job_name, callback):
        """Start getting detailed information about a jenkins job."""

        if not self.is_alive():
            logger.warning("Process is not running!")
        logger.debug("Got request for job details for job %s on server %s", job_name, server_url)
        self._enqueue_command(_GET_JOB_DATA,
                            [server_url, job_name],
                            callback)

    def _enqueue_command(self, cmd_id, params, callback):
        if type(cmd_id) is not int:
            raise TypeError("Command id must be an integer.")
        if type(params) is not list:
            raise TypeError("Command parameters must be a list.")
        if not callable(callback):
            raise TypeError("Callback parameter must be a callable object.")

        self._callback_queue.append(callback)
        self._input.put([cmd_id, params])


    def run(self):
        logger.debug("WORKER PROCESS STARTING.")
        while not self._quit_event.is_set():
            work_item = self._input.get()
            if isinstance(work_item, list):
                try:
                    command, args = work_item
                    logger.debug("Got command %d", command)
                    js = jenkins.Jenkins(args[0])
                    if command == _GET_SERVER_DATA:
                        data = js.get_jobs()
                    elif command == _GET_JOB_DATA:
                        data = js.get_job_info(args[1])
                    self._output.put([True, data])
                except Exception, e:
                    self._output.put([False, e])

    def _poll_for_data(self):
        # I may delete this poll and put this code into a thread. Seems nicer that
        # way, although there may be tricky bits mixing threads and multiple processes
        while not self._output.empty():
            data = self._output.get()
            callback = self._callback_queue.pop(0)
            logger.debug("Got reply, calling callback: %r", callback)
            callback(data)
        return True

    def stop(self):
        """Stop the worker process."""
        self._quit_event.set()
        self._input.put(False)
        self.join()

