# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# Copyright 2012 Thomi Richards

import json
import os
import os.path
import logging
logger = logging.getLogger(__name__)

_DEFAULTS = {
    "servers": {'default':
        {
        "url": "",
        "monitor_jobs": [],
        "refresh_period": 2 * 60,
        }
    }
}
_SETTINGS_INSTANCE = None

class SettingsStore(object):
    """A class responsible for loading and saving settings from/to disk."""

    _SETTINGS_FILE_DIR = os.path.join(os.path.expanduser('~'), '.config')
    _SETTINGS_FILE_PATH = os.path.join(_SETTINGS_FILE_DIR, 'indicator-jenkins')

    def __init__(self):
        self._load_settings()

    def _load_settings(self):
        """Load settings from disk, or load defaults."""
        try:
            if os.path.exists(self._SETTINGS_FILE_PATH):
                with open(self._SETTINGS_FILE_PATH, 'r') as f:
                    data = json.load(f)
                self._data = self._validate_data(data)
            else:
                logging.debug("No settings file to load.")
                self._data = self._get_defaults()
        except (IOError, ValueError):
            logging.debug("No settings file to load.")
            self._data = self._get_defaults()

    def _validate_data(self, raw_data):
        """Return a validated version of `raw_data`."""
        if type(raw_data) is not dict:
            logger.warning("Settings root object must be a dict. Recreating from defaults.")
            raw_data = _DEFAULTS

        if not raw_data.has_key("servers") \
            or type(raw_data['servers']) is not dict \
            or len(raw_data['servers']) == 0:
            logger.warning("Settngs have no 'server' section, creating from defaults.")
            raw_data['servers'] = _DEFAULTS['servers']

        # TODO: we can actually use server_data inside the for loop to set values...
        for name, server_data in raw_data['servers'].iteritems():
            if not (server_data.has_key('url') and isinstance(server_data['url'], basestring)):
                raw_data['servers'][name]['url'] = ""
            if not (server_data.has_key('monitor_jobs') and type(server_data['monitor_jobs']) is list):
                raw_data['servers'][name]['monitor_jobs'] = []
            if not (server_data.has_key('refresh_period') and type(server_data['refresh_period']) is int):
                raw_data['servers'][name]['refresh_period'] = 2 * 60
            if server_data['refresh_period'] < 30:
                raw_data['servers'][name]['refresh_period'] = 30
                logger.warning("Refresh period of %dS. for server '%s' is too low. Setting to 30S.",
                    server_data['refresh_period'], name)
            if server_data['refresh_period'] > 10*60:
                logger.warning("Refresh period of %dS. for server '%s' is too high. Setting to 600S.",
                    server_data['refresh_period'], name)
                raw_data['servers'][name]['refresh_period'] = 10*60
        return raw_data

    def _get_defaults(self):
        """Get the default settings."""
        return _DEFAULTS

    def get_all_servers(self):
        """Return a dictionary of name:server_dict for all configured servers."""
        return self._data['servers']

    def get_server_by_name(self, name):
        """Get a settings dict for the server with the given name."""
        return self._data['servers'][name]

    def update_settings_from_model(self, server_model):
        """Update a server in the settings store from a server model object."""
        self._data['servers'][server_model.name]['url'] = server_model.url
        self._data['servers'][server_model.name]['refresh_period'] = server_model.refresh_period
        mj = [j.name for j in server_model.get_jobs() if j.monitored]
        self._data['servers'][server_model.name]['monitor_jobs'] = mj

    def save_settings(self):
        """Save settings to disk."""
        logger.info("Saving settings to disk.")
        if not os.path.exists(self._SETTINGS_FILE_DIR):
            os.makedirs(self._SETTINGS_FILE_DIR)
        with open(self._SETTINGS_FILE_PATH, 'w') as f:
            json.dump(self._data, f, indent=4)

def get_settings():
    global _SETTINGS_INSTANCE
    if _SETTINGS_INSTANCE is None:
        _SETTINGS_INSTANCE = SettingsStore()
    return _SETTINGS_INSTANCE

def save_settings():
    global _SETTINGS_INSTANCE
    if _SETTINGS_INSTANCE is not None:
        _SETTINGS_INSTANCE.save_settings()
